pub struct TaskScope(/* private fields */);Implementations
Sourceimpl TaskScope
impl TaskScope
Sourcepub fn alloc<T>(&self, capacity: usize) -> Buffer<T, Self>
pub fn alloc<T>(&self, capacity: usize) -> Buffer<T, Self>
Allocates a buffer in this scope on the device.
This call is not blocking. Upon successful completion, it will return a buffer with a memory that is guaranteed to be available in the scope of the task but without any absolute guarantee relative to the host or any other task.
Other tasks may try to allocate memory concurrently. In order to guarantee enough memory for all expected work, the user must ensure some limit on task calls by e.g. using a semaphore.
Sourcepub fn try_alloc<T>(
&self,
capacity: usize,
) -> Result<Buffer<T, Self>, TryReserveError>
pub fn try_alloc<T>( &self, capacity: usize, ) -> Result<Buffer<T, Self>, TryReserveError>
Tries to allocate a buffer in this scope on the device.
Sourcepub unsafe fn launch_host_fn(
&self,
host_fn: unsafe extern "C" fn(*mut c_void),
data: *mut c_void,
) -> Result<(), CudaError>
pub unsafe fn launch_host_fn( &self, host_fn: unsafe extern "C" fn(*mut c_void), data: *mut c_void, ) -> Result<(), CudaError>
Launches a host function in this task.
Safety
The function essentially executes an extern call in C. The safety assumption of an extern.
The user must ensure the pointer is valid and that the data remains valid as this call will
be asynchronous.
Sourcepub unsafe fn launch_kernel(
&self,
kernel: KernelPtr,
grid_dim: impl Into<Dim3>,
block_dim: impl Into<Dim3>,
args: &[*mut c_void],
shared_mem: usize,
) -> Result<(), CudaError>
pub unsafe fn launch_kernel( &self, kernel: KernelPtr, grid_dim: impl Into<Dim3>, block_dim: impl Into<Dim3>, args: &[*mut c_void], shared_mem: usize, ) -> Result<(), CudaError>
Launches a kernel in this task.
Safety
The caller must ensure that:
- The kernel ptr is valid.
- The arguments are passed correctly across the FFI interface.
- The data lives whitin the scope of the current task.
Sourcepub fn sleep(&self, time: Duration)
pub fn sleep(&self, time: Duration)
Sends the CUDA task to sleep for at least the given duration.
This function will not block the calling host thread. The function does a small allocation so the sleep time might be slightly longer than the given duration for very short times.
Sourcepub unsafe fn copy<T: DeviceCopy>(
&self,
dst: &mut Slice<T, Self>,
src: &Slice<T, Self>,
) -> Result<(), CopyError>
pub unsafe fn copy<T: DeviceCopy>( &self, dst: &mut Slice<T, Self>, src: &Slice<T, Self>, ) -> Result<(), CopyError>
Copies data between slices using CudaMemCpyAsync
Safety
The caller must ensure that the data is valid and that the data remains valid as this call
Sourcepub async fn synchronize(&self) -> Result<(), CudaError>
pub async fn synchronize(&self) -> Result<(), CudaError>
Waits for all work enqueued so far in this task to finish.
This function can be useful in case there is work to be enqueued but for some reason this work cannot be done using Self::launch_host_fn.
Sourcepub fn into_device<T: IntoDevice>(
&self,
data: T,
) -> Result<T::Output, CopyError>
pub fn into_device<T: IntoDevice>( &self, data: T, ) -> Result<T::Output, CopyError>
Copies data from the host to the device.
pub fn to_device<T: ToDevice>(&self, data: &T) -> Result<T::Output, CopyError>
Sourcepub fn synchronize_blocking(&self) -> Result<(), CudaError>
pub fn synchronize_blocking(&self) -> Result<(), CudaError>
Waits for all work enqueued so far in this task to finish.
This function can be useful in case there is work to be enqueued but for some reason this work cannot be done using Self::launch_host_fn.
Sourcepub unsafe fn handle(&self) -> CudaStreamHandle
pub unsafe fn handle(&self) -> CudaStreamHandle
Safety
pub fn owner(&self) -> TaskPool
Sourcepub fn spawn<F, Fut>(&self, f: F) -> SpawnHandle<Fut::Output>
pub fn spawn<F, Fut>(&self, f: F) -> SpawnHandle<Fut::Output>
Spawns a new task from the current task pool.
The task starting point will have a “happens before” relationship with the current task when the spawn is called. The handle can be used to wait for the child task to finish.
Sourcepub async fn run_in_place<F, Fut>(&self, f: F) -> Result<Fut::Output, CudaError>
pub async fn run_in_place<F, Fut>(&self, f: F) -> Result<Fut::Output, CudaError>
Runs a task in place in a new stream.
Awaiting this task will peform the device calls and synchronize the end of this task to the parent, but does not do host synchronization.